@@ -49,8 +49,11 @@ import org.apache.activemq.artemis.core.config.amqpBrokerConnectivity.AMQPMirror
 import org.apache.activemq.artemis.core.postoffice.Binding;
 import org.apache.activemq.artemis.core.postoffice.QueueBinding;
 import org.apache.activemq.artemis.core.remoting.CertificateUtil;
+import org.apache.activemq.artemis.core.remoting.CloseListener;
+import org.apache.activemq.artemis.core.remoting.FailureListener;
 import org.apache.activemq.artemis.core.remoting.impl.netty.NettyConnection;
 import org.apache.activemq.artemis.core.remoting.impl.netty.NettyConnector;
+import org.apache.activemq.artemis.core.remoting.impl.netty.NettyConnectorFactory;
 import org.apache.activemq.artemis.core.remoting.impl.netty.TransportConstants;
 import org.apache.activemq.artemis.core.server.ActiveMQComponent;
 import org.apache.activemq.artemis.core.server.ActiveMQServer;
@@ -65,10 +68,15 @@ import org.apache.activemq.artemis.protocol.amqp.broker.AMQPMessage;
 import org.apache.activemq.artemis.protocol.amqp.broker.AMQPSessionCallback;
 import org.apache.activemq.artemis.protocol.amqp.broker.ActiveMQProtonRemotingConnection;
 import org.apache.activemq.artemis.protocol.amqp.broker.ProtonProtocolManager;
+import org.apache.activemq.artemis.protocol.amqp.broker.ProtonProtocolManagerFactory;
+import org.apache.activemq.artemis.protocol.amqp.connect.AMQPBrokerConnectionManager.ClientProtocolManagerWithAMQP;
 import org.apache.activemq.artemis.protocol.amqp.connect.federation.AMQPFederationPolicySupport;
 import org.apache.activemq.artemis.protocol.amqp.connect.federation.AMQPFederationSource;
 import org.apache.activemq.artemis.protocol.amqp.connect.mirror.AMQPMirrorControllerAggregation;
 import org.apache.activemq.artemis.protocol.amqp.connect.mirror.AMQPMirrorControllerSource;
+import org.apache.activemq.artemis.protocol.amqp.connect.mirror.ReferenceIDSupplier;
+import org.apache.activemq.artemis.protocol.amqp.exceptions.ActiveMQAMQPException;
+import org.apache.activemq.artemis.protocol.amqp.exceptions.ActiveMQAMQPInternalErrorException;
 import org.apache.activemq.artemis.protocol.amqp.logger.ActiveMQAMQPProtocolLogger;
 import org.apache.activemq.artemis.protocol.amqp.logger.ActiveMQAMQPProtocolMessageBundle;
 import org.apache.activemq.artemis.protocol.amqp.proton.AMQPLargeMessageWriter;
@@ -78,6 +86,7 @@ import org.apache.activemq.artemis.protocol.amqp.proton.AMQPTunneledCoreLargeMes
 import org.apache.activemq.artemis.protocol.amqp.proton.AMQPTunneledCoreMessageWriter;
 import org.apache.activemq.artemis.protocol.amqp.proton.AmqpSupport;
 import org.apache.activemq.artemis.protocol.amqp.proton.MessageWriter;
+import org.apache.activemq.artemis.protocol.amqp.proton.ProtonServerReceiverContext;
 import org.apache.activemq.artemis.protocol.amqp.proton.ProtonServerSenderContext;
 import org.apache.activemq.artemis.protocol.amqp.proton.SenderController;
 import org.apache.activemq.artemis.protocol.amqp.sasl.ClientSASL;
@@ -93,6 +102,7 @@ import org.apache.activemq.artemis.utils.UUIDGenerator;
 import org.apache.qpid.proton.amqp.Symbol;
 import org.apache.qpid.proton.amqp.messaging.Source;
 import org.apache.qpid.proton.amqp.messaging.Target;
+import org.apache.qpid.proton.amqp.transport.ErrorCondition;
 import org.apache.qpid.proton.amqp.transport.ReceiverSettleMode;
 import org.apache.qpid.proton.amqp.transport.SenderSettleMode;
 import org.apache.qpid.proton.engine.EndpointState;
@@ -120,10 +130,13 @@ public class AMQPBrokerConnection implements ClientConnectionLifeCycleListener,
     */
    public static final boolean DEFAULT_CORE_MESSAGE_TUNNELING_ENABLED = true;
 
+   private static final NettyConnectorFactory CONNECTOR_FACTORY = new NettyConnectorFactory().setServerConnector(true);
+
+   private final ProtonProtocolManagerFactory protonProtocolManagerFactory;
+   private final ReferenceIDSupplier referenceIdSupplier;
    private final AMQPBrokerConnectConfiguration brokerConnectConfiguration;
-   private final ProtonProtocolManager protonProtocolManager;
    private final ActiveMQServer server;
-   private final NettyConnector bridgesConnector;
+   private final List<TransportConfiguration> configurations;
    private NettyConnection connection;
    private Session session;
    private AMQPSessionContext sessionContext;
@@ -134,6 +147,7 @@ public class AMQPBrokerConnection implements ClientConnectionLifeCycleListener,
    private AMQPFederationSource brokerFederation;
    private int retryCounter = 0;
    private int lastRetryCounter;
+   private int connectionTimeout;
    private boolean connecting = false;
    private volatile ScheduledFuture<?> reconnectFuture;
    private final Set<Queue> senders = new HashSet<>();
@@ -153,16 +167,16 @@ public class AMQPBrokerConnection implements ClientConnectionLifeCycleListener,
 
    public AMQPBrokerConnection(AMQPBrokerConnectionManager bridgeManager,
                                AMQPBrokerConnectConfiguration brokerConnectConfiguration,
-                               ProtonProtocolManager protonProtocolManager,
-                               ActiveMQServer server,
-                               NettyConnector bridgesConnector) {
+                               ProtonProtocolManagerFactory protonProtocolManagerFactory,
+                               ActiveMQServer server) throws Exception {
       this.bridgeManager = bridgeManager;
       this.brokerConnectConfiguration = brokerConnectConfiguration;
-      this.protonProtocolManager = protonProtocolManager;
       this.server = server;
-      this.bridgesConnector = bridgesConnector;
-      connectExecutor = server.getExecutorFactory().getExecutor();
-      scheduledExecutorService = server.getScheduledPool();
+      this.configurations = brokerConnectConfiguration.getTransportConfigurations();
+      this.connectExecutor = server.getExecutorFactory().getExecutor();
+      this.scheduledExecutorService = server.getScheduledPool();
+      this.protonProtocolManagerFactory = protonProtocolManagerFactory;
+      this.referenceIdSupplier = new ReferenceIDSupplier(server);
    }
 
    @Override
@@ -190,7 +204,7 @@ public class AMQPBrokerConnection implements ClientConnectionLifeCycleListener,
    }
 
    public int getConnectionTimeout() {
-      return bridgesConnector.getConnectTimeoutMillis();
+      return connectionTimeout;
    }
 
    @Override
@@ -274,20 +288,17 @@ public class AMQPBrokerConnection implements ClientConnectionLifeCycleListener,
          Symbol[] dispatchCapability = new Symbol[]{AMQPMirrorControllerSource.QPID_DISPATCH_WAYPOINT_CAPABILITY};
          connectSender(queue, queue.getAddress().toString(), null, null, null, null, dispatchCapability, null);
          connectReceiver(protonRemotingConnection, session, sessionContext, queue, dispatchCapability);
-      } else {
-         if (connectionElement.getType() == AMQPBrokerConnectionAddressType.SENDER) {
-            connectSender(queue, queue.getAddress().toString(), null, null, null, null, null, null);
-         }
-         if (connectionElement.getType() == AMQPBrokerConnectionAddressType.RECEIVER) {
-            connectReceiver(protonRemotingConnection, session, sessionContext, queue);
-         }
+      } else if (connectionElement.getType() == AMQPBrokerConnectionAddressType.SENDER) {
+         connectSender(queue, queue.getAddress().toString(), null, null, null, null, null, null);
+      } else if (connectionElement.getType() == AMQPBrokerConnectionAddressType.RECEIVER) {
+         connectReceiver(protonRemotingConnection, session, sessionContext, queue);
       }
    }
 
    SimpleString getMirrorSNF(AMQPMirrorBrokerConnectionElement mirrorElement) {
       SimpleString snf = mirrorElement.getMirrorSNF();
       if (snf == null) {
-         snf = SimpleString.toSimpleString(ProtonProtocolManager.getMirrorAddress(this.brokerConnectConfiguration.getName()));
+         snf = SimpleString.of(ProtonProtocolManager.getMirrorAddress(this.brokerConnectConfiguration.getName()));
          mirrorElement.setMirrorSNF(snf);
       }
       return snf;
@@ -340,19 +351,32 @@ public class AMQPBrokerConnection implements ClientConnectionLifeCycleListener,
       try {
          connecting = true;
 
-         List<TransportConfiguration> configurationList = brokerConnectConfiguration.getTransportConfigurations();
+         TransportConfiguration configuration = configurations.get(retryCounter % configurations.size());
+         host = ConfigurationHelper.getStringProperty(TransportConstants.HOST_PROP_NAME, TransportConstants.DEFAULT_HOST, configuration.getParams());
+         port = ConfigurationHelper.getIntProperty(TransportConstants.PORT_PROP_NAME, TransportConstants.DEFAULT_PORT, configuration.getParams());
 
-         TransportConfiguration tpConfig = configurationList.get(retryCounter % configurationList.size());
+         ProtonProtocolManager protonProtocolManager =
+            (ProtonProtocolManager)protonProtocolManagerFactory.createProtocolManager(server, configuration.getExtraParams(), null, null);
+         NettyConnector connector = (NettyConnector)CONNECTOR_FACTORY.createConnector(
+            configuration.getParams(), null, this, server.getExecutorFactory().getExecutor(), server.getThreadPool(), server.getScheduledPool(), new ClientProtocolManagerWithAMQP(protonProtocolManager));
+         connector.start();
 
-         String hostOnParameter = ConfigurationHelper.getStringProperty(TransportConstants.HOST_PROP_NAME, TransportConstants.DEFAULT_HOST, tpConfig.getParams());
-         int portOnParameter = ConfigurationHelper.getIntProperty(TransportConstants.PORT_PROP_NAME, TransportConstants.DEFAULT_PORT, tpConfig.getParams());
-         this.host = hostOnParameter;
-         this.port = portOnParameter;
-         connection = bridgesConnector.createConnection(null, hostOnParameter, portOnParameter);
+         logger.debug("Connecting {}", configuration);
 
-         if (connection == null) {
-            retryConnection();
-            return;
+         connectionTimeout = connector.getConnectTimeoutMillis();
+         try {
+            connection = (NettyConnection) connector.createConnection();
+            if (connection == null) {
+               retryConnection();
+               return;
+            }
+         } finally {
+            if (connection == null) {
+               try {
+                  connector.close();
+               } catch (Exception ex) {
+               }
+            }
          }
 
          lastRetryCounter = retryCounter;
@@ -368,12 +392,15 @@ public class AMQPBrokerConnection implements ClientConnectionLifeCycleListener,
 
          ClientSASLFactory saslFactory = new SaslFactory(connection, brokerConnectConfiguration);
 
+         NettyConnectorCloseHandler connectorCloseHandler = new NettyConnectorCloseHandler(connector, connectExecutor);
          ConnectionEntry entry = protonProtocolManager.createOutgoingConnectionEntry(connection, saslFactory);
          server.getRemotingService().addConnectionEntry(connection, entry);
          protonRemotingConnection = (ActiveMQProtonRemotingConnection) entry.connection;
          protonRemotingConnection.getAmqpConnection().addLinkRemoteCloseListener(getName(), this::linkClosed);
+         protonRemotingConnection.addCloseListener(connectorCloseHandler);
+         protonRemotingConnection.addFailureListener(connectorCloseHandler);
 
-         connection.getChannel().pipeline().addLast(new AMQPBrokerConnectionChannelHandler(bridgesConnector.getChannelGroup(), protonRemotingConnection.getAmqpConnection().getHandler(), this, server.getExecutorFactory().getExecutor()));
+         connection.getChannel().pipeline().addLast(new AMQPBrokerConnectionChannelHandler(connector.getChannelGroup(), protonRemotingConnection.getAmqpConnection().getHandler(), this, server.getExecutorFactory().getExecutor()));
 
          session = protonRemotingConnection.getAmqpConnection().getHandler().getConnection().session();
          sessionContext = protonRemotingConnection.getAmqpConnection().getSessionExtension(session);
@@ -505,11 +532,11 @@ public class AMQPBrokerConnection implements ClientConnectionLifeCycleListener,
       Queue mirrorControlQueue = server.locateQueue(getMirrorSNF(replicaConfig));
 
       if (mirrorControlQueue == null) {
-         mirrorControlQueue = server.createQueue(new QueueConfiguration(getMirrorSNF(replicaConfig)).setAddress(getMirrorSNF(replicaConfig)).setRoutingType(RoutingType.ANYCAST).setDurable(replicaConfig.isDurable()).setInternal(true), true);
+         mirrorControlQueue = server.createQueue(QueueConfiguration.of(getMirrorSNF(replicaConfig)).setAddress(getMirrorSNF(replicaConfig)).setRoutingType(RoutingType.ANYCAST).setDurable(replicaConfig.isDurable()).setInternal(true), true);
       }
 
       try {
-         server.registerQueueOnManagement(mirrorControlQueue, true);
+         server.registerQueueOnManagement(mirrorControlQueue);
       } catch (Throwable ignored) {
          logger.debug(ignored.getMessage(), ignored);
       }
@@ -531,7 +558,7 @@ public class AMQPBrokerConnection implements ClientConnectionLifeCycleListener,
          throw new IllegalAccessException("Cannot start replica");
       }
 
-      AMQPMirrorControllerSource newPartition = new AMQPMirrorControllerSource(protonProtocolManager, snfQueue, server, replicaConfig, this);
+      AMQPMirrorControllerSource newPartition = new AMQPMirrorControllerSource(referenceIdSupplier, snfQueue, server, replicaConfig, this);
 
       this.mirrorControllerSource = newPartition;
 
@@ -618,34 +645,105 @@ public class AMQPBrokerConnection implements ClientConnectionLifeCycleListener,
       }
 
       protonRemotingConnection.getAmqpConnection().runLater(() -> {
-
-         if (receivers.contains(queue)) {
+         if (!receivers.add(queue)) {
             logger.debug("Receiver for queue {} already exists, just giving up", queue);
             return;
          }
-         receivers.add(queue);
-         Receiver receiver = session.receiver(queue.getAddress().toString() + ":" + UUIDGenerator.getInstance().generateStringUUID());
-         receiver.setSenderSettleMode(SenderSettleMode.UNSETTLED);
-         receiver.setReceiverSettleMode(ReceiverSettleMode.FIRST);
-         Target target = new Target();
-         target.setAddress(queue.getAddress().toString());
-         receiver.setTarget(target);
 
-         Source source = new Source();
-         source.setAddress(queue.getAddress().toString());
-         receiver.setSource(source);
-
-         if (capabilities != null) {
-            source.setCapabilities(capabilities);
-         }
-
-         receiver.open();
-         protonRemotingConnection.getAmqpConnection().flush();
          try {
-            sessionContext.addReceiver(receiver);
+            final String linkName = queue.getAddress().toString() + ":" + UUIDGenerator.getInstance().generateStringUUID();
+            final Receiver receiver = session.receiver(linkName);
+            final String queueAddress = queue.getAddress().toString();
+
+            final Target target = new Target();
+            target.setAddress(queueAddress);
+            final Source source = new Source();
+            source.setAddress(queueAddress);
+            if (capabilities != null) {
+               source.setCapabilities(capabilities);
+            }
+
+            receiver.setSenderSettleMode(SenderSettleMode.UNSETTLED);
+            receiver.setReceiverSettleMode(ReceiverSettleMode.FIRST);
+            receiver.setTarget(target);
+            receiver.setSource(source);
+            receiver.open();
+
+            final ScheduledFuture<?> openTimeoutTask;
+            final AtomicBoolean openTimedOut = new AtomicBoolean(false);
+
+            if (getConnectionTimeout() > 0) {
+               openTimeoutTask = server.getScheduledPool().schedule(() -> {
+                  openTimedOut.set(true);
+                  error(ActiveMQAMQPProtocolMessageBundle.BUNDLE.brokerConnectionTimeout(), lastRetryCounter);
+               }, getConnectionTimeout(), TimeUnit.MILLISECONDS);
+            } else {
+               openTimeoutTask = null;
+            }
+
+            // Await the remote attach before creating the broker receiver in order to impose a timeout
+            // on the attach response and then try and create the local server receiver context and finish
+            // the wiring.
+            receiver.attachments().set(AMQP_LINK_INITIALIZER_KEY, Runnable.class, () -> {
+               try {
+                  if (openTimeoutTask != null) {
+                     openTimeoutTask.cancel(false);
+                  }
+
+                  if (openTimedOut.get()) {
+                     return; // Timed out before remote attach arrived
+                  }
+
+                  if (receiver.getRemoteSource() != null) {
+                     logger.trace("AMQP Broker Connection Receiver {} completed open", linkName);
+                  } else {
+                     logger.debug("AMQP Broker Connection Receiver {} rejected by remote", linkName);
+                     error(ActiveMQAMQPProtocolMessageBundle.BUNDLE.receiverLinkRefused(queueAddress), lastRetryCounter);
+                     return;
+                  }
+
+                  sessionContext.addReceiver(receiver, (r, s) -> {
+                     // Returns a customized server receiver context that will respect the locally initiated state
+                     // when the receiver is initialized vs the remotely sent target as we want to ensure we attach
+                     // the receiver to the address we set in our local state.
+                     return new ProtonServerReceiverContext(sessionContext.getSessionSPI(),
+                                                            sessionContext.getAMQPConnectionContext(),
+                                                            sessionContext, receiver) {
+
+                        @Override
+                        public void initialize() throws Exception {
+                           initialized = true;
+                           address = SimpleString.of(target.getAddress());
+                           defRoutingType = getRoutingType(target.getCapabilities(), address);
+
+                           try {
+                              // Check if the queue that triggered the attach still exists or has it been removed
+                              // before the attach response arrived from the remote peer.
+                              if (!sessionSPI.queueQuery(queue.getName(), queue.getRoutingType(), false).isExists()) {
+                                 throw ActiveMQAMQPProtocolMessageBundle.BUNDLE.addressDoesntExist(address.toString());
+                              }
+                           } catch (ActiveMQAMQPException e) {
+                              receivers.remove(queue);
+                              throw e;
+                           } catch (Exception e) {
+                              logger.debug(e.getMessage(), e);
+                              receivers.remove(queue);
+                              throw new ActiveMQAMQPInternalErrorException(e.getMessage(), e);
+                           }
+
+                           flow();
+                        }
+                     };
+                  });
+               } catch (Exception e) {
+                  error(e);
+               }
+            });
          } catch (Exception e) {
             error(e);
          }
+
+         protonRemotingConnection.getAmqpConnection().flush();
       });
    }
 
@@ -702,11 +800,11 @@ public class AMQPBrokerConnection implements ClientConnectionLifeCycleListener,
 
             AtomicBoolean cancelled = new AtomicBoolean(false);
 
-            if (bridgesConnector.getConnectTimeoutMillis() > 0) {
+            if (getConnectionTimeout() > 0) {
                futureTimeout = server.getScheduledPool().schedule(() -> {
                   cancelled.set(true);
                   error(ActiveMQAMQPProtocolMessageBundle.BUNDLE.brokerConnectionTimeout(), lastRetryCounter);
-               }, bridgesConnector.getConnectTimeoutMillis(), TimeUnit.MILLISECONDS);
+               }, getConnectionTimeout(), TimeUnit.MILLISECONDS);
             } else {
                futureTimeout = null;
             }
@@ -836,6 +934,18 @@ public class AMQPBrokerConnection implements ClientConnectionLifeCycleListener,
 
       @Override
       public void close() throws Exception {
+
+      }
+
+      @Override
+      public void close(ErrorCondition error) {
+         // If remote closed already than normal broker connection link closed handling will already kick in,
+         // if not then the connection should be locally force closed as it would otherwise sit in a zombie state
+         // never consuming from the SNF Queue again.
+         if (sender.getRemoteState() != EndpointState.CLOSED) {
+            AMQPBrokerConnection.this.runtimeError(new ActiveMQAMQPInternalErrorException(
+               "Broker connection mirror consumer locally closed unexpectedly: " + error.getCondition().toString()));
+         }
       }
 
       @Override
@@ -1059,4 +1169,39 @@ public class AMQPBrokerConnection implements ClientConnectionLifeCycleListener,
          return DEFAULT_CORE_MESSAGE_TUNNELING_ENABLED;
       }
    }
+
+   public static class NettyConnectorCloseHandler implements FailureListener, CloseListener {
+
+      private final NettyConnector connector;
+      private final Executor connectionExecutor;
+
+      public NettyConnectorCloseHandler(NettyConnector connector, Executor connectionExecutor) {
+         this.connector = connector;
+         this.connectionExecutor = connectionExecutor;
+      }
+
+      @Override
+      public void connectionClosed() {
+         doCloseConnector();
+      }
+
+      @Override
+      public void connectionFailed(ActiveMQException exception, boolean failedOver) {
+         doCloseConnector();
+      }
+
+      @Override
+      public void connectionFailed(ActiveMQException exception, boolean failedOver, String scaleDownTargetNodeID) {
+         doCloseConnector();
+      }
+
+      private void doCloseConnector() {
+         connectionExecutor.execute(() -> {
+            try {
+               connector.close();
+            } catch (Exception ex) {
+            }
+         });
+      }
+   }
 }
