@@ -22,7 +22,9 @@ import static org.apache.activemq.artemis.protocol.amqp.connect.federation.AMQPF
 import static org.apache.activemq.artemis.protocol.amqp.connect.federation.AMQPFederationConstants.ADDRESS_AUTO_DELETE_MSG_COUNT;
 import static org.apache.activemq.artemis.protocol.amqp.connect.federation.AMQPFederationConstants.FEDERATION_ADDRESS_RECEIVER;
 import static org.apache.activemq.artemis.protocol.amqp.connect.federation.AMQPFederationPolicySupport.FEDERATED_ADDRESS_SOURCE_PROPERTIES;
-
+import static org.apache.activemq.artemis.protocol.amqp.connect.federation.AMQPFederationPolicySupport.MESSAGE_HOPS_PROPERTY;
+import static org.apache.activemq.artemis.protocol.amqp.proton.AMQPTunneledMessageConstants.AMQP_TUNNELED_CORE_LARGE_MESSAGE_FORMAT;
+import static org.apache.activemq.artemis.protocol.amqp.proton.AMQPTunneledMessageConstants.AMQP_TUNNELED_CORE_MESSAGE_FORMAT;
 import static org.apache.activemq.artemis.protocol.amqp.proton.AmqpSupport.AMQP_LINK_INITIALIZER_KEY;
 import static org.apache.activemq.artemis.protocol.amqp.proton.AmqpSupport.DETACH_FORCED;
 import static org.apache.activemq.artemis.protocol.amqp.proton.AmqpSupport.NOT_FOUND;
@@ -39,6 +41,7 @@ import java.util.function.Consumer;
 import java.util.function.Predicate;
 
 import org.apache.activemq.artemis.api.core.ActiveMQException;
+import org.apache.activemq.artemis.api.core.ICoreMessage;
 import org.apache.activemq.artemis.api.core.Message;
 import org.apache.activemq.artemis.api.core.RoutingType;
 import org.apache.activemq.artemis.api.core.SimpleString;
@@ -59,10 +62,16 @@ import org.apache.activemq.artemis.protocol.amqp.logger.ActiveMQAMQPProtocolMess
 import org.apache.activemq.artemis.protocol.amqp.proton.AMQPConnectionContext;
 import org.apache.activemq.artemis.protocol.amqp.proton.AMQPSessionContext;
 import org.apache.activemq.artemis.protocol.amqp.proton.AmqpJmsSelectorFilter;
+import org.apache.activemq.artemis.protocol.amqp.proton.AmqpNoLocalFilter;
 import org.apache.activemq.artemis.protocol.amqp.proton.AmqpSupport;
+import org.apache.activemq.artemis.protocol.amqp.proton.AMQPTunneledCoreLargeMessageReader;
+import org.apache.activemq.artemis.protocol.amqp.proton.AMQPTunneledCoreMessageReader;
+import org.apache.activemq.artemis.protocol.amqp.proton.MessageReader;
 import org.apache.activemq.artemis.protocol.amqp.proton.ProtonServerReceiverContext;
+import org.apache.activemq.artemis.reader.MessageUtil;
 import org.apache.qpid.proton.amqp.Symbol;
 import org.apache.qpid.proton.amqp.messaging.Accepted;
+import org.apache.qpid.proton.amqp.messaging.DeliveryAnnotations;
 import org.apache.qpid.proton.amqp.messaging.Modified;
 import org.apache.qpid.proton.amqp.messaging.Rejected;
 import org.apache.qpid.proton.amqp.messaging.Released;
@@ -89,11 +98,7 @@ public class AMQPFederationAddressConsumer implements FederationConsumerInternal
 
    // Redefined because AMQPMessage uses SimpleString in its annotations API for some reason.
    private static final SimpleString MESSAGE_HOPS_ANNOTATION =
-      new SimpleString(AMQPFederationPolicySupport.MESSAGE_HOPS_ANNOTATION.toString());
-
-   // Desired capabilities that the federation receiver link needs the remote to offer in order
-   // for the federation receiver to be successfully opened.
-   private static final Symbol[] DESIRED_LINK_CAPABILITIES = new Symbol[] {FEDERATION_ADDRESS_RECEIVER};
+      SimpleString.of(AMQPFederationPolicySupport.MESSAGE_HOPS_ANNOTATION.toString());
 
    private static final Symbol[] DEFAULT_OUTCOMES = new Symbol[]{Accepted.DESCRIPTOR_SYMBOL, Rejected.DESCRIPTOR_SYMBOL,
                                                                  Released.DESCRIPTOR_SYMBOL, Modified.DESCRIPTOR_SYMBOL};
@@ -162,7 +167,7 @@ public class AMQPFederationAddressConsumer implements FederationConsumerInternal
          if (started) {
             started = false;
             connection.runLater(() -> {
-               federation.removeLinkClosedInterceptor(consumerInfo.getFqqn());
+               federation.removeLinkClosedInterceptor(consumerInfo.getId());
 
                if (receiver != null) {
                   try {
@@ -271,18 +276,20 @@ public class AMQPFederationAddressConsumer implements FederationConsumerInternal
                source.setCapabilities(AmqpSupport.TOPIC_CAPABILITY);
             }
 
+            final Map<Symbol, Object> filtersMap = new HashMap<>();
+            filtersMap.put(AmqpSupport.NO_LOCAL_NAME, AmqpNoLocalFilter.NO_LOCAL);
+
+            if (consumerInfo.getFilterString() != null && !consumerInfo.getFilterString().isEmpty()) {
+               final AmqpJmsSelectorFilter jmsFilter = new AmqpJmsSelectorFilter(consumerInfo.getFilterString());
+
+               filtersMap.put(AmqpSupport.JMS_SELECTOR_KEY, jmsFilter);
+            }
+
             source.setOutcomes(Arrays.copyOf(DEFAULT_OUTCOMES, DEFAULT_OUTCOMES.length));
             source.setDurable(TerminusDurability.NONE);
             source.setExpiryPolicy(TerminusExpiryPolicy.LINK_DETACH);
             source.setAddress(address);
-
-            if (consumerInfo.getFilterString() != null && !consumerInfo.getFilterString().isEmpty()) {
-               final AmqpJmsSelectorFilter jmsFilter = new AmqpJmsSelectorFilter(consumerInfo.getFilterString());
-               final Map<Symbol, Object> filtersMap = new HashMap<>();
-               filtersMap.put(AmqpSupport.JMS_SELECTOR_KEY, jmsFilter);
-
-               source.setFilter(filtersMap);
-            }
+            source.setFilter(filtersMap);
 
             target.setAddress(address);
 
@@ -298,7 +305,14 @@ public class AMQPFederationAddressConsumer implements FederationConsumerInternal
 
             protonReceiver.setSenderSettleMode(SenderSettleMode.UNSETTLED);
             protonReceiver.setReceiverSettleMode(ReceiverSettleMode.FIRST);
-            protonReceiver.setDesiredCapabilities(DESIRED_LINK_CAPABILITIES);
+            protonReceiver.setDesiredCapabilities(new Symbol[] {FEDERATION_ADDRESS_RECEIVER});
+            // If enabled offer core tunneling which we prefer to AMQP conversions of core as
+            // the large ones will be converted to standard AMQP messages in memory. When not
+            // offered the remote must not use core tunneling and AMQP conversion will be the
+            // fallback.
+            if (configuration.isCoreMessageTunnelingEnabled()) {
+               protonReceiver.setOfferedCapabilities(new Symbol[] {AmqpSupport.CORE_MESSAGE_TUNNELING_SUPPORT});
+            }
             protonReceiver.setProperties(receiverProperties);
             protonReceiver.setTarget(target);
             protonReceiver.setSource(source);
@@ -307,11 +321,11 @@ public class AMQPFederationAddressConsumer implements FederationConsumerInternal
             final ScheduledFuture<?> openTimeoutTask;
             final AtomicBoolean openTimedOut = new AtomicBoolean(false);
 
-            if (federation.getLinkAttachTimeout() > 0) {
+            if (configuration.getLinkAttachTimeout() > 0) {
                openTimeoutTask = federation.getServer().getScheduledPool().schedule(() -> {
                   openTimedOut.set(true);
                   federation.signalResourceCreateError(ActiveMQAMQPProtocolMessageBundle.BUNDLE.brokerConnectionTimeout());
-               }, federation.getLinkAttachTimeout(), TimeUnit.SECONDS);
+               }, configuration.getLinkAttachTimeout(), TimeUnit.SECONDS);
             } else {
                openTimeoutTask = null;
             }
@@ -331,16 +345,16 @@ public class AMQPFederationAddressConsumer implements FederationConsumerInternal
                   // Remote must support federation receivers otherwise we fail the connection unless the
                   // Attach indicates that a detach is incoming in which case we just allow the normal handling
                   // to occur.
-                  if (protonReceiver.getRemoteSource() != null && !AmqpSupport.verifyOfferedCapabilities(protonReceiver)) {
+                  if (protonReceiver.getRemoteSource() != null && !AmqpSupport.verifyOfferedCapabilities(protonReceiver, FEDERATION_ADDRESS_RECEIVER)) {
                      federation.signalResourceCreateError(
-                        ActiveMQAMQPProtocolMessageBundle.BUNDLE.missingOfferedCapability(Arrays.toString(DESIRED_LINK_CAPABILITIES)));
+                        ActiveMQAMQPProtocolMessageBundle.BUNDLE.missingOfferedCapability(FEDERATION_ADDRESS_RECEIVER.toString()));
                      return;
                   }
 
                   // Intercept remote close and check for valid reasons for remote closure such as
                   // the remote peer not having a matching queue for this subscription or from an
                   // operator manually closing the link.
-                  federation.addLinkClosedInterceptor(consumerInfo.getFqqn(), remoteCloseInterceptor);
+                  federation.addLinkClosedInterceptor(consumerInfo.getId(), remoteCloseInterceptor);
 
                   receiver = new AMQPFederatedAddressDeliveryReceiver(session, consumerInfo, protonReceiver);
 
@@ -365,8 +379,9 @@ public class AMQPFederationAddressConsumer implements FederationConsumerInternal
       });
    }
 
-   private static AMQPMessage incrementMessageHops(AMQPMessage message) {
+   private static AMQPMessage incrementAMQPMessageHops(AMQPMessage message) {
       Object hops = message.getAnnotation(MESSAGE_HOPS_ANNOTATION);
+
       if (hops == null) {
          message.setAnnotation(MESSAGE_HOPS_ANNOTATION, 1);
       } else {
@@ -380,6 +395,19 @@ public class AMQPFederationAddressConsumer implements FederationConsumerInternal
       return message;
    }
 
+   private static ICoreMessage incrementCoreMessageHops(ICoreMessage message) {
+      Object hops = message.getObjectProperty(MESSAGE_HOPS_PROPERTY);
+
+      if (hops == null) {
+         message.putObjectProperty(MESSAGE_HOPS_PROPERTY, 1);
+      } else {
+         Number numHops = (Number) hops;
+         message.putObjectProperty(MESSAGE_HOPS_PROPERTY, numHops.intValue() + 1);
+      }
+
+      return message;
+   }
+
    /**
     * Wrapper around the standard receiver context that provides federation specific entry
     * points and customizes inbound delivery handling for this Address receiver.
@@ -388,6 +416,10 @@ public class AMQPFederationAddressConsumer implements FederationConsumerInternal
 
       private final SimpleString cachedAddress;
 
+      private MessageReader coreMessageReader;
+
+      private MessageReader coreLargeMessageReader;
+
       /**
        * Creates the federation receiver instance.
        *
@@ -399,7 +431,7 @@ public class AMQPFederationAddressConsumer implements FederationConsumerInternal
       AMQPFederatedAddressDeliveryReceiver(AMQPSessionContext session, FederationConsumerInfo consumerInfo, Receiver receiver) {
          super(session.getSessionSPI(), session.getAMQPConnectionContext(), session, receiver);
 
-         this.cachedAddress = SimpleString.toSimpleString(consumerInfo.getAddress());
+         this.cachedAddress = SimpleString.of(consumerInfo.getAddress());
       }
 
       @Override
@@ -452,7 +484,7 @@ public class AMQPFederationAddressConsumer implements FederationConsumerInternal
             throw new ActiveMQAMQPInternalErrorException("Remote should have sent an valid Target but we got: " + target);
          }
 
-         address = SimpleString.toSimpleString(target.getAddress());
+         address = SimpleString.of(target.getAddress());
          defRoutingType = getRoutingType(target.getCapabilities(), address);
 
          try {
@@ -474,18 +506,44 @@ public class AMQPFederationAddressConsumer implements FederationConsumerInternal
       }
 
       @Override
-      protected void actualDelivery(AMQPMessage message, Delivery delivery, Receiver receiver, Transaction tx) {
+      protected MessageReader trySelectMessageReader(Receiver receiver, Delivery delivery) {
+         if (delivery.getMessageFormat() == AMQP_TUNNELED_CORE_MESSAGE_FORMAT) {
+            return coreMessageReader != null ?
+               coreMessageReader : (coreMessageReader = new AMQPTunneledCoreMessageReader(this));
+         } else if (delivery.getMessageFormat() == AMQP_TUNNELED_CORE_LARGE_MESSAGE_FORMAT) {
+            return coreLargeMessageReader != null ?
+               coreLargeMessageReader : (coreLargeMessageReader = new AMQPTunneledCoreLargeMessageReader(this));
+         } else {
+            return super.trySelectMessageReader(receiver, delivery);
+         }
+      }
+
+      @Override
+      protected void actualDelivery(Message message, Delivery delivery, DeliveryAnnotations deliveryAnnotations, Receiver receiver, Transaction tx) {
          try {
             if (logger.isTraceEnabled()) {
                logger.trace("AMQP Federation {} address consumer {} dispatching incoming message: {}",
                             federation.getName(), consumerInfo, message);
             }
 
-            final Message theMessage = transformer.transform(incrementMessageHops(message));
+            final Message baseMessage;
 
-            if (theMessage != message && logger.isTraceEnabled()) {
+            if (message instanceof ICoreMessage) {
+               baseMessage = incrementCoreMessageHops((ICoreMessage) message);
+
+               // Add / Update the connection Id value to reflect the remote container Id so that the
+               // no-local filter of a federation address receiver directed back to the source of this
+               // message will exclude it as intended.
+               baseMessage.putStringProperty(MessageUtil.CONNECTION_ID_PROPERTY_NAME_STRING, getConnection().getRemoteContainer());
+            } else {
+               baseMessage = incrementAMQPMessageHops((AMQPMessage) message);
+            }
+
+            final Message theMessage = transformer.transform(baseMessage);
+
+            if (theMessage != baseMessage && logger.isTraceEnabled()) {
                logger.trace("The transformer {} replaced the original message {} with a new instance {}",
-                            transformer, message, theMessage);
+                            transformer, baseMessage, theMessage);
             }
 
             signalBeforeFederationConsumerMessageHandled(theMessage);
