@@ -16,76 +16,74 @@
  */
 package org.apache.activemq.artemis.core.protocol.stomp;
 
-import javax.security.cert.X509Certificate;
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.EnumSet;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
 import java.util.StringTokenizer;
-import java.util.concurrent.CopyOnWriteArrayList;
+import java.util.concurrent.Future;
+import java.util.concurrent.FutureTask;
 import java.util.concurrent.ScheduledExecutorService;
 
+import org.apache.activemq.artemis.api.core.ActiveMQAddressDoesNotExistException;
 import org.apache.activemq.artemis.api.core.ActiveMQBuffer;
-import org.apache.activemq.artemis.api.core.ActiveMQBuffers;
 import org.apache.activemq.artemis.api.core.ActiveMQException;
 import org.apache.activemq.artemis.api.core.ActiveMQQueueExistsException;
+import org.apache.activemq.artemis.api.core.ActiveMQSecurityException;
 import org.apache.activemq.artemis.api.core.ICoreMessage;
+import org.apache.activemq.artemis.api.core.QueueConfiguration;
 import org.apache.activemq.artemis.api.core.RoutingType;
 import org.apache.activemq.artemis.api.core.SimpleString;
 import org.apache.activemq.artemis.api.core.client.ActiveMQClient;
 import org.apache.activemq.artemis.core.message.impl.CoreMessage;
 import org.apache.activemq.artemis.core.protocol.stomp.v10.StompFrameHandlerV10;
 import org.apache.activemq.artemis.core.protocol.stomp.v12.StompFrameHandlerV12;
-import org.apache.activemq.artemis.core.remoting.CloseListener;
 import org.apache.activemq.artemis.core.remoting.FailureListener;
 import org.apache.activemq.artemis.core.remoting.impl.netty.TransportConstants;
 import org.apache.activemq.artemis.core.server.ActiveMQServerLogger;
+import org.apache.activemq.artemis.core.server.ServerConsumer;
 import org.apache.activemq.artemis.core.server.ServerSession;
+import org.apache.activemq.artemis.core.server.impl.AddressInfo;
 import org.apache.activemq.artemis.core.settings.impl.AddressSettings;
-import org.apache.activemq.artemis.spi.core.protocol.RemotingConnection;
+import org.apache.activemq.artemis.spi.core.protocol.AbstractRemotingConnection;
 import org.apache.activemq.artemis.spi.core.remoting.Acceptor;
 import org.apache.activemq.artemis.spi.core.remoting.Connection;
 import org.apache.activemq.artemis.spi.core.remoting.ReadyListener;
+import org.apache.activemq.artemis.utils.CompositeAddress;
 import org.apache.activemq.artemis.utils.ConfigurationHelper;
 import org.apache.activemq.artemis.utils.ExecutorFactory;
 import org.apache.activemq.artemis.utils.VersionLoader;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+import java.lang.invoke.MethodHandles;
 
 import static org.apache.activemq.artemis.core.protocol.stomp.ActiveMQStompProtocolMessageBundle.BUNDLE;
+import static org.apache.activemq.artemis.reader.MessageUtil.CONNECTION_ID_PROPERTY_NAME_STRING;
 
-public final class StompConnection implements RemotingConnection {
+public final class StompConnection extends AbstractRemotingConnection {
+
+   private static final Logger logger = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
 
-   protected static final String CONNECTION_ID_PROP = "__AMQ_CID";
    private static final String SERVER_NAME = "ActiveMQ-Artemis/" + VersionLoader.getVersion().getFullVersion() +
       " ActiveMQ Artemis Messaging Engine";
 
    private final StompProtocolManager manager;
 
-   private final Connection transportConnection;
-
    private String login;
 
    private String passcode;
 
-   private String clientID;
-
    //this means login is valid. (stomp connection ok)
    private boolean valid;
 
    private boolean destroyed = false;
 
-   private final long creationTime;
-
    private final Acceptor acceptorUsed;
 
-   private final List<FailureListener> failureListeners = new CopyOnWriteArrayList<>();
-
-   private final List<CloseListener> closeListeners = new CopyOnWriteArrayList<>();
-
    private final Object failLock = new Object();
 
-   private boolean dataReceived;
-
    private final boolean enableMessageID;
 
    private final int minLargeMessageSize;
@@ -103,17 +101,13 @@ public final class StompConnection implements RemotingConnection {
 
    private final ScheduledExecutorService scheduledExecutorService;
 
-   private final ExecutorFactory factory;
-
-   @Override
-   public boolean isSupportReconnect() {
-      return false;
-   }
+   private final ExecutorFactory executorFactory;
 
    public VersionedStompFrameHandler getStompVersionHandler() {
       return frameHandler;
    }
 
+
    public StompFrame decode(ActiveMQBuffer buffer) throws ActiveMQStompException {
       StompFrame frame = null;
       try {
@@ -123,7 +117,7 @@ public final class StompConnection implements RemotingConnection {
             case ActiveMQStompException.INVALID_EOL_V10:
                if (version != null)
                   throw e;
-               frameHandler = new StompFrameHandlerV12(this, scheduledExecutorService, factory);
+               frameHandler = new StompFrameHandlerV12(this, scheduledExecutorService, executorFactory);
                buffer.resetReaderIndex();
                frame = decode(buffer);
                break;
@@ -140,7 +134,7 @@ public final class StompConnection implements RemotingConnection {
 
    @Override
    public boolean isWritable(ReadyListener callback) {
-      return transportConnection.isWritable(callback);
+      return transportConnection.isWritable(callback) && transportConnection.isOpen();
    }
 
    public boolean hasBytes() {
@@ -151,175 +145,91 @@ public final class StompConnection implements RemotingConnection {
                    final Connection transportConnection,
                    final StompProtocolManager manager,
                    final ScheduledExecutorService scheduledExecutorService,
-                   final ExecutorFactory factory) {
+                   final ExecutorFactory executorFactory) {
+      super(transportConnection, null);
+
       this.scheduledExecutorService = scheduledExecutorService;
 
-      this.factory = factory;
-
-      this.transportConnection = transportConnection;
+      this.executorFactory = executorFactory;
 
       this.manager = manager;
 
-      this.frameHandler = new StompFrameHandlerV10(this, scheduledExecutorService, factory);
-
-      this.creationTime = System.currentTimeMillis();
+      this.frameHandler = new StompFrameHandlerV10(this, scheduledExecutorService, executorFactory);
 
       this.acceptorUsed = acceptorUsed;
 
-      this.enableMessageID = ConfigurationHelper.getBooleanProperty(TransportConstants.STOMP_ENABLE_MESSAGE_ID, false, acceptorUsed.getConfiguration());
-      this.minLargeMessageSize = ConfigurationHelper.getIntProperty(TransportConstants.STOMP_MIN_LARGE_MESSAGE_SIZE, ActiveMQClient.DEFAULT_MIN_LARGE_MESSAGE_SIZE, acceptorUsed.getConfiguration());
-   }
-
-   @Override
-   public void addFailureListener(final FailureListener listener) {
-      if (listener == null) {
-         throw new IllegalStateException("FailureListener cannot be null");
-      }
-
-      failureListeners.add(listener);
-   }
-
-   @Override
-   public boolean removeFailureListener(final FailureListener listener) {
-      if (listener == null) {
-         throw new IllegalStateException("FailureListener cannot be null");
-      }
-
-      return failureListeners.remove(listener);
-   }
-
-   @Override
-   public void addCloseListener(final CloseListener listener) {
-      if (listener == null) {
-         throw new IllegalStateException("CloseListener cannot be null");
-      }
-
-      closeListeners.add(listener);
-   }
-
-   @Override
-   public boolean removeCloseListener(final CloseListener listener) {
-      if (listener == null) {
-         throw new IllegalStateException("CloseListener cannot be null");
-      }
-
-      return closeListeners.remove(listener);
-   }
-
-   @Override
-   public List<CloseListener> removeCloseListeners() {
-      List<CloseListener> ret = new ArrayList<>(closeListeners);
-
-      closeListeners.clear();
-
-      return ret;
-   }
-
-   @Override
-   public List<FailureListener> removeFailureListeners() {
-      List<FailureListener> ret = new ArrayList<>(failureListeners);
-
-      failureListeners.clear();
-
-      return ret;
-   }
-
-   @Override
-   public void setCloseListeners(List<CloseListener> listeners) {
-      closeListeners.clear();
-
-      closeListeners.addAll(listeners);
-   }
-
-   @Override
-   public void setFailureListeners(final List<FailureListener> listeners) {
-      failureListeners.clear();
-
-      failureListeners.addAll(listeners);
-   }
-
-   protected synchronized void setDataReceived() {
-      dataReceived = true;
-   }
-
-   @Override
-   public synchronized boolean checkDataReceived() {
-      boolean res = dataReceived;
-
-      dataReceived = false;
-
-      return res;
+      this.enableMessageID = ConfigurationHelper.getBooleanProperty(TransportConstants.STOMP_ENABLE_MESSAGE_ID_DEPRECATED, false, acceptorUsed.getConfiguration()) || ConfigurationHelper.getBooleanProperty(TransportConstants.STOMP_ENABLE_MESSAGE_ID, false, acceptorUsed.getConfiguration());
+      this.minLargeMessageSize = ConfigurationHelper.getIntProperty(TransportConstants.STOMP_MIN_LARGE_MESSAGE_SIZE, ConfigurationHelper.getIntProperty(TransportConstants.STOMP_MIN_LARGE_MESSAGE_SIZE_DEPRECATED, ActiveMQClient.DEFAULT_MIN_LARGE_MESSAGE_SIZE, acceptorUsed.getConfiguration()), acceptorUsed.getConfiguration());
    }
 
    // TODO this should take a type - send or receive so it knows whether to check the address or the queue
    public void checkDestination(String destination) throws ActiveMQStompException {
-      if (!manager.destinationExists(getSession().getCoreSession().removePrefix(SimpleString.toSimpleString(destination)).toString())) {
+      if (!manager.destinationExists(destination)) {
          throw BUNDLE.destinationNotExist(destination).setHandler(frameHandler);
       }
    }
 
-   public boolean autoCreateDestinationIfPossible(String queue, RoutingType routingType) throws ActiveMQStompException {
-      boolean result = false;
-      ServerSession session = getSession().getCoreSession();
-
+   public void autoCreateDestinationIfPossible(String destination, RoutingType routingType) throws ActiveMQStompException {
       try {
-         if (manager.getServer().getAddressInfo(SimpleString.toSimpleString(queue)) == null) {
-            AddressSettings addressSettings = manager.getServer().getAddressSettingsRepository().getMatch(queue);
-            if (routingType != null && routingType == RoutingType.MULTICAST && addressSettings.isAutoCreateAddresses()) {
-               session.createAddress(SimpleString.toSimpleString(queue), RoutingType.MULTICAST, true);
-               result = true;
-            } else {
-               if (addressSettings.isAutoCreateAddresses()) {
-                  session.createAddress(SimpleString.toSimpleString(queue), RoutingType.ANYCAST, true);
-                  result = true;
-               }
-               if (addressSettings.isAutoCreateQueues()) {
-                  session.createQueue(SimpleString.toSimpleString(queue), SimpleString.toSimpleString(queue), RoutingType.ANYCAST, null, false, true, true);
-                  result = true;
-               }
+         SimpleString simpleDestination = SimpleString.of(destination);
+         AddressInfo addressInfo = manager.getServer().getAddressInfo(simpleDestination);
+         AddressSettings addressSettings = manager.getServer().getAddressSettingsRepository().getMatch(destination);
+         RoutingType effectiveAddressRoutingType = routingType == null ? addressSettings.getDefaultAddressRoutingType() : routingType;
+         ServerSession session = getSession().getCoreSession();
+         /**
+          * If the address doesn't exist then it is created if possible.
+          * If the address does exist but doesn't support the routing-type then the address is updated if possible.
+          */
+         if (addressInfo == null) {
+            if (addressSettings.isAutoCreateAddresses()) {
+               session.createAddress(simpleDestination, effectiveAddressRoutingType, true);
             }
+         } else if (!addressInfo.getRoutingTypes().contains(effectiveAddressRoutingType)) {
+            if (addressSettings.isAutoCreateAddresses()) {
+               EnumSet<RoutingType> routingTypes = EnumSet.noneOf(RoutingType.class);
+               for (RoutingType existingRoutingType : addressInfo.getRoutingTypes()) {
+                  routingTypes.add(existingRoutingType);
+               }
+               routingTypes.add(effectiveAddressRoutingType);
+               manager.getServer().updateAddressInfo(simpleDestination, routingTypes);
+            }
+         }
+
+         // auto create the queue if the address is ANYCAST or FQQN
+         if ((CompositeAddress.isFullyQualified(destination) || effectiveAddressRoutingType == RoutingType.ANYCAST) && addressSettings.isAutoCreateQueues() && manager.getServer().locateQueue(simpleDestination) == null) {
+            session.createQueue(QueueConfiguration.of(destination).setRoutingType(effectiveAddressRoutingType).setAutoCreated(true));
          }
       } catch (ActiveMQQueueExistsException e) {
          // ignore
       } catch (Exception e) {
+         logger.debug("Exception while auto-creating destination", e);
          throw new ActiveMQStompException(e.getMessage(), e).setHandler(frameHandler);
       }
-
-      return result;
    }
 
    public void checkRoutingSemantics(String destination, RoutingType routingType) throws ActiveMQStompException {
-      Set<RoutingType> actualDeliveryModesOfAddress = manager.getServer().getAddressInfo(getSession().getCoreSession().removePrefix(SimpleString.toSimpleString(destination))).getRoutingTypes();
-      if (routingType != null && !actualDeliveryModesOfAddress.contains(routingType)) {
-         throw BUNDLE.illegalSemantics(routingType.toString(), actualDeliveryModesOfAddress.toString());
+      AddressInfo addressInfo = manager.getServer().getAddressInfo(SimpleString.of(destination));
+
+      // may be null here if, for example, the management address is being checked
+      if (addressInfo != null) {
+         Set<RoutingType> actualDeliveryModesOfAddress = addressInfo.getRoutingTypes();
+         if (routingType != null && !actualDeliveryModesOfAddress.contains(routingType)) {
+            throw BUNDLE.illegalSemantics(routingType.toString(), actualDeliveryModesOfAddress.toString());
+         }
       }
    }
 
-   @Override
-   public ActiveMQBuffer createTransportBuffer(int size) {
-      return createTransportBuffer(size, false);
-   }
-
-   @Override
-   public ActiveMQBuffer createTransportBuffer(int size, boolean pooled) {
-      return ActiveMQBuffers.dynamicBuffer(size);
-   }
-
    @Override
    public void destroy() {
       synchronized (failLock) {
          if (destroyed) {
             return;
          }
-      }
 
-      destroyed = true;
+         destroyed = true;
+      }
 
       internalClose();
-
-      synchronized (sendLock) {
-         callClosingListeners();
-      }
    }
 
    public Acceptor getAcceptorUsed() {
@@ -327,9 +237,17 @@ public final class StompConnection implements RemotingConnection {
    }
 
    private void internalClose() {
+      if (frameHandler != null) {
+         frameHandler.disconnect();
+      }
+
       transportConnection.close();
 
       manager.cleanup(this);
+
+      synchronized (sendLock) {
+         callClosingListeners();
+      }
    }
 
    @Override
@@ -341,32 +259,40 @@ public final class StompConnection implements RemotingConnection {
 
          StompFrame frame = frameHandler.createStompFrame(Stomp.Responses.ERROR);
          frame.addHeader(Stomp.Headers.Error.MESSAGE, me.getMessage());
-         sendFrame(frame);
+         sendFrame(frame, null);
 
          destroyed = true;
       }
 
       ActiveMQServerLogger.LOGGER.connectionFailureDetected(me.getMessage(), me.getType());
 
-      if (frameHandler != null) {
-         frameHandler.disconnect();
-      }
-
       // Then call the listeners
       callFailureListeners(me);
 
-      callClosingListeners();
-
       internalClose();
    }
 
    @Override
-   public void fail(final ActiveMQException me, String scaleDownTargetNodeID) {
-      fail(me);
+   public Future asyncFail(ActiveMQException me) {
+
+      FutureTask<Void> task = new FutureTask(() -> {
+         fail(me);
+         return null;
+      });
+
+      if (this.executorFactory == null) {
+         // only tests cases can do this
+         task.run();
+      } else {
+         executorFactory.getExecutor().execute(task);
+      }
+
+      return task;
    }
 
    @Override
-   public void flush() {
+   public void fail(final ActiveMQException me, String scaleDownTargetNodeID) {
+      fail(me);
    }
 
    @Override
@@ -376,38 +302,9 @@ public final class StompConnection implements RemotingConnection {
       return Collections.emptyList();
    }
 
-   @Override
-   public Object getID() {
-      return transportConnection.getID();
-   }
-
-   @Override
-   public String getRemoteAddress() {
-      return transportConnection.getRemoteAddress();
-   }
-
-   @Override
-   public long getCreationTime() {
-      return creationTime;
-   }
-
-   @Override
-   public Connection getTransportConnection() {
-      return transportConnection;
-   }
-
-   @Override
-   public boolean isClient() {
-      return false;
-   }
-
-   @Override
-   public boolean isDestroyed() {
-      return destroyed;
-   }
-
    @Override
    public void bufferReceived(Object connectionID, ActiveMQBuffer buffer) {
+      super.bufferReceived(connectionID, buffer);
       manager.handleBuffer(this, buffer);
    }
 
@@ -415,16 +312,16 @@ public final class StompConnection implements RemotingConnection {
       return login;
    }
 
+   public void setLogin(String login) {
+      this.login = login;
+   }
+
    public String getPasscode() {
       return passcode;
    }
 
-   public void setClientID(String clientID) {
-      this.clientID = clientID;
-   }
-
-   public String getClientID() {
-      return clientID;
+   public void setPasscode(String passcode) {
+      this.passcode = passcode;
    }
 
    public boolean isValid() {
@@ -442,24 +339,7 @@ public final class StompConnection implements RemotingConnection {
          try {
             listener.connectionFailed(me, false);
          } catch (final Throwable t) {
-            // Failure of one listener to execute shouldn't prevent others
-            // from
-            // executing
-            ActiveMQServerLogger.LOGGER.errorCallingFailureListener(t);
-         }
-      }
-   }
-
-   private void callClosingListeners() {
-      final List<CloseListener> listenersClone = new ArrayList<>(closeListeners);
-
-      for (final CloseListener listener : listenersClone) {
-         try {
-            listener.connectionClosed();
-         } catch (final Throwable t) {
-            // Failure of one listener to execute shouldn't prevent others
-            // from
-            // executing
+            // Failure of one listener to execute shouldn't prevent others from executing
             ActiveMQServerLogger.LOGGER.errorCallingFailureListener(t);
          }
       }
@@ -499,7 +379,7 @@ public final class StompConnection implements RemotingConnection {
       }
 
       if (this.version != (StompVersions.V1_0)) {
-         VersionedStompFrameHandler newHandler = VersionedStompFrameHandler.getHandler(this, this.version, scheduledExecutorService, factory);
+         VersionedStompFrameHandler newHandler = VersionedStompFrameHandler.getHandler(this, this.version, scheduledExecutorService, executorFactory);
          newHandler.initDecoder(this.frameHandler);
          this.frameHandler = newHandler;
       }
@@ -548,7 +428,7 @@ public final class StompConnection implements RemotingConnection {
       }
 
       if (reply != null) {
-         sendFrame(reply);
+         sendFrame(reply, null);
       }
 
       if (Stomp.Commands.DISCONNECT.equals(cmd)) {
@@ -556,28 +436,40 @@ public final class StompConnection implements RemotingConnection {
       }
    }
 
-   public void sendFrame(StompFrame frame) {
-      manager.sendReply(this, frame);
+   public void logFrame(StompFrame request, boolean in) {
+      if (logger.isDebugEnabled()) {
+         StringBuilder message = new StringBuilder()
+            .append("STOMP(")
+            .append(getRemoteAddress())
+            .append(", ")
+            .append(this.getID())
+            .append("):");
+
+         if (in) {
+            message.append(" IN << ");
+         } else {
+            message.append("OUT >> ");
+         }
+
+         message.append(request);
+
+         logger.debug(message.toString());
+      }
    }
 
-   public boolean validateUser(final String login, final String pass, final X509Certificate[] certificates) {
-      this.valid = manager.validateUser(login, pass, certificates);
-      if (valid) {
-         this.login = login;
-         this.passcode = pass;
-      }
-      return valid;
+   public void sendFrame(StompFrame frame, StompPostReceiptFunction function) {
+      manager.sendReply(this, frame, function);
    }
 
    public CoreMessage createServerMessage() {
       return manager.createServerMessage();
    }
 
-   public StompSession getSession() throws ActiveMQStompException {
+   public StompSession getSession() throws ActiveMQStompException, ActiveMQSecurityException {
       return getSession(null);
    }
 
-   public StompSession getSession(String txID) throws ActiveMQStompException {
+   public StompSession getSession(String txID) throws ActiveMQStompException, ActiveMQSecurityException {
       StompSession session = null;
       try {
          if (txID == null) {
@@ -585,6 +477,8 @@ public final class StompConnection implements RemotingConnection {
          } else {
             session = manager.getTransactedSession(this, txID);
          }
+      } catch (ActiveMQSecurityException e) {
+         throw e;
       } catch (Exception e) {
          throw BUNDLE.errorGetSession(e).setHandler(frameHandler);
       }
@@ -599,15 +493,15 @@ public final class StompConnection implements RemotingConnection {
    }
 
    protected void sendServerMessage(ICoreMessage message, String txID) throws ActiveMQStompException {
-      StompSession stompSession = getSession(txID);
-
-      if (stompSession.isNoLocal()) {
-         message.putStringProperty(CONNECTION_ID_PROP, getID().toString());
-      }
-      if (isEnableMessageID()) {
-         message.putStringProperty("amqMessageId", "STOMP" + message.getMessageID());
-      }
       try {
+         StompSession stompSession = getSession(txID);
+
+         if (stompSession.isNoLocal()) {
+            message.putStringProperty(CONNECTION_ID_PROPERTY_NAME_STRING, getID().toString());
+         }
+         if (isEnableMessageID()) {
+            message.putStringProperty("amqMessageId", "STOMP" + message.getMessageID());
+         }
          if (minLargeMessageSize == -1 || (message.getBodyBuffer().writerIndex() < minLargeMessageSize)) {
             stompSession.sendInternal(message, false);
          } else {
@@ -656,18 +550,19 @@ public final class StompConnection implements RemotingConnection {
       }
    }
 
-   void subscribe(String destination,
-                  String selector,
-                  String ack,
-                  String id,
-                  String durableSubscriptionName,
-                  boolean noLocal,
-                  RoutingType subscriptionType) throws ActiveMQStompException {
+   StompPostReceiptFunction subscribe(String destination,
+                                      String selector,
+                                      String ack,
+                                      String id,
+                                      String durableSubscriptionName,
+                                      boolean noLocal,
+                                      RoutingType subscriptionType,
+                                      Integer consumerWindowSize) throws ActiveMQStompException {
       autoCreateDestinationIfPossible(destination, subscriptionType);
       checkDestination(destination);
       checkRoutingSemantics(destination, subscriptionType);
       if (noLocal) {
-         String noLocalFilter = CONNECTION_ID_PROP + " <> '" + getID().toString() + "'";
+         String noLocalFilter = CONNECTION_ID_PROPERTY_NAME_STRING + " <> '" + getID().toString() + "'";
          if (selector == null) {
             selector = noLocalFilter;
          } else {
@@ -690,7 +585,7 @@ public final class StompConnection implements RemotingConnection {
       }
 
       try {
-         manager.subscribe(this, subscriptionID, durableSubscriptionName, destination, selector, ack, noLocal);
+         return manager.subscribe(this, subscriptionID, durableSubscriptionName, destination, selector, ack, noLocal, consumerWindowSize);
       } catch (ActiveMQStompException e) {
          throw e;
       } catch (Exception e) {
@@ -701,6 +596,8 @@ public final class StompConnection implements RemotingConnection {
    public void unsubscribe(String subscriptionID, String durableSubscriptionName) throws ActiveMQStompException {
       try {
          manager.unsubscribe(this, subscriptionID, durableSubscriptionName);
+      } catch (ActiveMQAddressDoesNotExistException e) {
+         // this could happen if multiple clients unsubscribe simultaneously and auto-delete-addresses = true
       } catch (ActiveMQStompException e) {
          throw e;
       } catch (Exception e) {
@@ -726,11 +623,11 @@ public final class StompConnection implements RemotingConnection {
       return SERVER_NAME;
    }
 
-   public StompFrame createStompMessage(ICoreMessage serverMessage,
-                                        ActiveMQBuffer bodyBuffer,
+   public StompFrame createStompMessage(ICoreMessage message,
                                         StompSubscription subscription,
-                                        int deliveryCount) throws Exception {
-      return frameHandler.createMessageFrame(serverMessage, bodyBuffer, subscription, deliveryCount);
+                                        ServerConsumer consumer,
+                                        int deliveryCount) throws ActiveMQException {
+      return frameHandler.createMessageFrame(message, subscription, consumer, deliveryCount);
    }
 
    public void addStompEventListener(FrameEventListener listener) {
@@ -739,7 +636,7 @@ public final class StompConnection implements RemotingConnection {
 
    //send a ping stomp frame
    public void ping(StompFrame pingFrame) {
-      manager.sendReply(this, pingFrame);
+      manager.sendReply(this, pingFrame, null);
    }
 
    public void physicalSend(StompFrame frame) throws Exception {
@@ -752,6 +649,13 @@ public final class StompConnection implements RemotingConnection {
          stompListener.replySent(frame);
       }
 
+      if (frame.getCommand().equals(Stomp.Responses.ERROR)) {
+         String message = "no message header";
+         if (frame.hasHeader(Stomp.Headers.Error.MESSAGE)) {
+            message = frame.getHeader(Stomp.Headers.Error.MESSAGE);
+         }
+         ActiveMQStompProtocolLogger.LOGGER.sentErrorToClient(getTransportConnection().getRemoteAddress(), message);
+      }
    }
 
    public VersionedStompFrameHandler getFrameHandler() {
@@ -770,13 +674,13 @@ public final class StompConnection implements RemotingConnection {
       return manager;
    }
 
+   /**
+    * Returns the name of the protocol for this Remoting Connection
+    *
+    * @return
+    */
    @Override
-   public void killMessage(SimpleString nodeID) {
-      //unsupported
-   }
-
-   @Override
-   public boolean isSupportsFlowControl() {
-      return false;
+   public String getProtocolName() {
+      return StompProtocolManagerFactory.STOMP_PROTOCOL_NAME;
    }
 }
