@@ -393,7 +393,7 @@ public final class PagingManagerImpl implements PagingManager {
             if (oldStore != null) {
                oldStore.stop();
             }
-            store.getCursorProvider().counterRebuildStarted(); // TODO-NOW-DONT-MERGE maybe this should be removed
+            store.getCursorProvider().counterRebuildStarted();
             store.start();
             stores.put(store.getStoreName(), store);
          }
@@ -405,16 +405,17 @@ public final class PagingManagerImpl implements PagingManager {
 
    @Override
    public void deletePageStore(final SimpleString storeName) throws Exception {
+      PagingStore store;
       syncLock.readLock().lock();
       try {
-         PagingStore store = stores.remove(CompositeAddress.extractAddressName(storeName));
-         if (store != null) {
-            store.stop();
-            store.destroy();
-         }
+         store = stores.remove(CompositeAddress.extractAddressName(storeName));
       } finally {
          syncLock.readLock().unlock();
       }
+
+      if (store != null) {
+         store.destroy();
+      }
    }
 
    /**
