@@ -35,9 +35,7 @@ import java.util.concurrent.Executor;
 
 import org.apache.activemq.artemis.api.core.ActiveMQException;
 import org.apache.activemq.artemis.api.core.Interceptor;
-import org.apache.activemq.artemis.api.core.Message;
 import org.apache.activemq.artemis.api.core.SimpleString;
-import org.apache.activemq.artemis.core.config.Configuration;
 import org.apache.activemq.artemis.core.io.SequentialFile;
 import org.apache.activemq.artemis.core.journal.EncoderPersister;
 import org.apache.activemq.artemis.core.journal.Journal;
@@ -298,7 +296,6 @@ public final class ReplicationEndpoint implements ChannelHandler, ActiveMQCompon
 
    @Override
    public synchronized void start() throws Exception {
-      Configuration config = server.getConfiguration();
       try {
          storageManager = server.getStorageManager();
          storageManager.start();
@@ -309,7 +306,7 @@ public final class ReplicationEndpoint implements ChannelHandler, ActiveMQCompon
          journalsHolder.put(JournalContent.MESSAGES, storageManager.getMessageJournal());
 
          for (JournalContent jc : EnumSet.allOf(JournalContent.class)) {
-            filesReservedForSync.put(jc, new HashMap<Long, JournalSyncFile>());
+            filesReservedForSync.put(jc, new HashMap<>());
             // We only need to load internal structures on the backup...
             journalLoadInformation[jc.typeByte] = journalsHolder.get(jc).loadSyncOnly(JournalState.SYNCING);
          }
@@ -636,17 +633,14 @@ public final class ReplicationEndpoint implements ChannelHandler, ActiveMQCompon
             }
             message.releaseResources(true, false);
          } else {
-            executor.execute(new Runnable() {
-               @Override
-               public void run() {
-                  try {
-                     if (logger.isTraceEnabled()) {
-                        logger.trace("Deleting LargeMessage {} on the executor @ handleLargeMessageEnd", packet.getMessageId());
-                     }
-                     message.deleteFile();
-                  } catch (Exception e) {
-                     ActiveMQServerLogger.LOGGER.errorDeletingLargeMessage(packet.getMessageId(), e);
+            executor.execute(() -> {
+               try {
+                  if (logger.isTraceEnabled()) {
+                     logger.trace("Deleting LargeMessage {} on the executor @ handleLargeMessageEnd", packet.getMessageId());
                   }
+                  message.deleteFile();
+               } catch (Exception e) {
+                  ActiveMQServerLogger.LOGGER.errorDeletingLargeMessage(packet.getMessageId(), e);
                }
             });
          }
@@ -836,8 +830,7 @@ public final class ReplicationEndpoint implements ChannelHandler, ActiveMQCompon
    private void handlePageWrite(final ReplicationPageWriteMessage packet) throws Exception {
       PagedMessage pgdMessage = packet.getPagedMessage();
       pgdMessage.initMessage(storageManager);
-      Message msg = pgdMessage.getMessage();
-      Page page = getPage(msg.getAddressSimpleString(), packet.getPageNumber());
+      Page page = getPage(packet.getAddress(), packet.getPageNumber());
       page.writeDirect(pgdMessage);
    }
 
