@@ -709,12 +709,6 @@ public class FakeQueue extends CriticalComponentImpl implements Queue {
       return id;
    }
 
-   @Override
-   public MessageReference getReference(final long id1) {
-      // no-op
-      return null;
-   }
-
    @Override
    public int getScheduledCount() {
       // no-op
@@ -1029,4 +1023,8 @@ public class FakeQueue extends CriticalComponentImpl implements Queue {
       return 0;
    }
 
+   @Override
+   public int getInitialQueueBufferSize() {
+      return 0;
+   }
 }
