@@ -93,7 +93,7 @@ public class ClusteredMirrorSoakTest extends SoakTestBase {
    private static String DC2_NODEA_URI = "tcp://localhost:61618";
    private static String DC2_NODEB_URI = "tcp://localhost:61619";
 
-   private static void createServer(String serverName, String connectionName, String clusterURI, String mirrorURI, int porOffset, boolean paging) throws Exception {
+   private static void createServer(String serverName, String connectionName, String clusterURI, String mirrorURI, int portOffset, boolean paging) throws Exception {
       File serverLocation = getFileServerLocation(serverName);
       deleteDirectory(serverLocation);
 
@@ -106,7 +106,7 @@ public class ClusteredMirrorSoakTest extends SoakTestBase {
       cliCreateServer.setArgs("--no-stomp-acceptor", "--no-hornetq-acceptor", "--no-mqtt-acceptor", "--no-amqp-acceptor", "--max-hops", "1", "--name", DC1_NODE_A);
       cliCreateServer.addArgs("--addresses", "order");
       cliCreateServer.addArgs("--queues", "myQueue");
-      cliCreateServer.setPortOffset(porOffset);
+      cliCreateServer.setPortOffset(portOffset);
       cliCreateServer.createServer();
 
       Properties brokerProperties = new Properties();
