@@ -17,6 +17,11 @@
 
 package org.apache.activemq.artemis.tests.soak.brokerConnection.mirror;
 
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertNull;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+
 import javax.jms.Connection;
 import javax.jms.ConnectionFactory;
 import javax.jms.MessageConsumer;
@@ -24,6 +29,7 @@ import javax.jms.MessageProducer;
 import javax.jms.Session;
 import javax.jms.TextMessage;
 import javax.jms.Topic;
+
 import java.io.File;
 import java.io.StringWriter;
 import java.lang.invoke.MethodHandles;
@@ -48,9 +54,8 @@ import org.apache.activemq.artemis.utils.TestParameters;
 import org.apache.activemq.artemis.utils.Wait;
 import org.apache.activemq.artemis.utils.actors.OrderedExecutor;
 import org.apache.activemq.artemis.utils.cli.helper.HelperCreate;
-import org.junit.After;
-import org.junit.Assert;
-import org.junit.Test;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.Test;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -98,7 +103,7 @@ public class SingleMirrorSoakTest extends SoakTestBase {
    volatile Process processDC1;
    volatile Process processDC2;
 
-   @After
+   @AfterEach
    public void destroyServers() throws Exception {
       if (processDC1 != null) {
          processDC1.destroyForcibly();
@@ -119,7 +124,7 @@ public class SingleMirrorSoakTest extends SoakTestBase {
    private static void createServer(String serverName,
                                     String connectionName,
                                     String mirrorURI,
-                                    int porOffset,
+                                    int portOffset,
                                     boolean paging) throws Exception {
       File serverLocation = getFileServerLocation(serverName);
       deleteDirectory(serverLocation);
@@ -131,7 +136,7 @@ public class SingleMirrorSoakTest extends SoakTestBase {
       cliCreateServer.setNoWeb(false);
       cliCreateServer.setArgs("--no-stomp-acceptor", "--no-hornetq-acceptor", "--no-mqtt-acceptor", "--no-amqp-acceptor", "--max-hops", "1", "--name", DC1_NODE);
       cliCreateServer.addArgs("--addresses", TOPIC_NAME);
-      cliCreateServer.setPortOffset(porOffset);
+      cliCreateServer.setPortOffset(portOffset);
       cliCreateServer.createServer();
 
       Properties brokerProperties = new Properties();
@@ -157,13 +162,13 @@ public class SingleMirrorSoakTest extends SoakTestBase {
       saveProperties(brokerProperties, brokerPropertiesFile);
 
       File brokerXml = new File(serverLocation, "/etc/broker.xml");
-      Assert.assertTrue(brokerXml.exists());
+      assertTrue(brokerXml.exists());
       // Adding redistribution delay to broker configuration
-      Assert.assertTrue(FileUtil.findReplace(brokerXml, "<address-setting match=\"#\">", "<address-setting match=\"#\">\n\n" + "            <redistribution-delay>0</redistribution-delay> <!-- added by SimpleMirrorSoakTest.java --> \n"));
+      assertTrue(FileUtil.findReplace(brokerXml, "<address-setting match=\"#\">", "<address-setting match=\"#\">\n\n" + "            <redistribution-delay>0</redistribution-delay> <!-- added by SimpleMirrorSoakTest.java --> \n"));
 
       if (TRACE_LOGS) {
          File log4j = new File(serverLocation, "/etc/log4j2.properties");
-         Assert.assertTrue(FileUtil.findReplace(log4j, "logger.artemis_utils.level=INFO", "logger.artemis_utils.level=INFO\n" +
+         assertTrue(FileUtil.findReplace(log4j, "logger.artemis_utils.level=INFO", "logger.artemis_utils.level=INFO\n" +
             "\n" + "logger.ack.name=org.apache.activemq.artemis.protocol.amqp.connect.mirror.AckManager\n"
             + "logger.ack.level=TRACE\n"
             + "logger.config.name=org.apache.activemq.artemis.core.config.impl.ConfigurationImpl\n"
@@ -199,7 +204,7 @@ public class SingleMirrorSoakTest extends SoakTestBase {
       startServers();
 
 
-      Assert.assertTrue(KILL_INTERVAL > SEND_COMMIT || KILL_INTERVAL < 0);
+      assertTrue(KILL_INTERVAL > SEND_COMMIT || KILL_INTERVAL < 0);
 
       String clientIDA = "nodeA";
       String clientIDB = "nodeB";
@@ -217,10 +222,10 @@ public class SingleMirrorSoakTest extends SoakTestBase {
       runAfter(() -> managementDC1.close());
       runAfter(() -> managementDC2.close());
 
-      Wait.assertEquals(0, () -> getCount(managementDC1, clientIDA + "." + subscriptionID));
-      Wait.assertEquals(0, () -> getCount(managementDC2, clientIDA + "." + subscriptionID));
-      Wait.assertEquals(0, () -> getCount(managementDC1, clientIDB + "." + subscriptionID));
-      Wait.assertEquals(0, () -> getCount(managementDC2, clientIDB + "." + subscriptionID));
+      Wait.assertEquals(0, () -> getMessageCount(managementDC1, clientIDA + "." + subscriptionID));
+      Wait.assertEquals(0, () -> getMessageCount(managementDC2, clientIDA + "." + subscriptionID));
+      Wait.assertEquals(0, () -> getMessageCount(managementDC1, clientIDB + "." + subscriptionID));
+      Wait.assertEquals(0, () -> getMessageCount(managementDC2, clientIDB + "." + subscriptionID));
 
       ExecutorService executorService = Executors.newFixedThreadPool(3);
       runAfter(executorService::shutdownNow);
@@ -284,12 +289,12 @@ public class SingleMirrorSoakTest extends SoakTestBase {
 
       consumerDone.await(SNF_TIMEOUT, TimeUnit.MILLISECONDS);
 
-      Wait.assertEquals(0, () -> getCount(managementDC1, snfQueue), SNF_TIMEOUT);
-      Wait.assertEquals(0, () -> getCount(managementDC2, snfQueue), SNF_TIMEOUT);
-      Wait.assertEquals(NUMBER_MESSAGES - NUMBER_MESSAGES_RECEIVE, () -> getCount(managementDC1, clientIDA + "." + subscriptionID), GENERAL_WAIT_TIMEOUT);
-      Wait.assertEquals(NUMBER_MESSAGES - NUMBER_MESSAGES_RECEIVE, () -> getCount(managementDC1, clientIDB + "." + subscriptionID), GENERAL_WAIT_TIMEOUT);
-      Wait.assertEquals(NUMBER_MESSAGES - NUMBER_MESSAGES_RECEIVE, () -> getCount(managementDC2, clientIDA + "." + subscriptionID), GENERAL_WAIT_TIMEOUT);
-      Wait.assertEquals(NUMBER_MESSAGES - NUMBER_MESSAGES_RECEIVE, () -> getCount(managementDC2, clientIDB + "." + subscriptionID), GENERAL_WAIT_TIMEOUT);
+      Wait.assertEquals(0, () -> getMessageCount(managementDC1, snfQueue), SNF_TIMEOUT);
+      Wait.assertEquals(0, () -> getMessageCount(managementDC2, snfQueue), SNF_TIMEOUT);
+      Wait.assertEquals(NUMBER_MESSAGES - NUMBER_MESSAGES_RECEIVE, () -> getMessageCount(managementDC1, clientIDA + "." + subscriptionID), GENERAL_WAIT_TIMEOUT);
+      Wait.assertEquals(NUMBER_MESSAGES - NUMBER_MESSAGES_RECEIVE, () -> getMessageCount(managementDC1, clientIDB + "." + subscriptionID), GENERAL_WAIT_TIMEOUT);
+      Wait.assertEquals(NUMBER_MESSAGES - NUMBER_MESSAGES_RECEIVE, () -> getMessageCount(managementDC2, clientIDA + "." + subscriptionID), GENERAL_WAIT_TIMEOUT);
+      Wait.assertEquals(NUMBER_MESSAGES - NUMBER_MESSAGES_RECEIVE, () -> getMessageCount(managementDC2, clientIDB + "." + subscriptionID), GENERAL_WAIT_TIMEOUT);
 
       destroyServers();
 
@@ -303,9 +308,9 @@ public class SingleMirrorSoakTest extends SoakTestBase {
       server.getStorageManager().getMessageJournal().scheduleCompactAndBlock(10_000);
       HashMap<Integer, AtomicInteger> records = countJournal(server.getConfiguration());
       AtomicInteger duplicateRecordsCount = records.get((int) JournalRecordIds.DUPLICATE_ID);
-      Assert.assertNotNull(duplicateRecordsCount);
+      assertNotNull(duplicateRecordsCount);
       // 1000 credits by default
-      Assert.assertTrue(duplicateRecordsCount.get() <= 1000);
+      assertTrue(duplicateRecordsCount.get() <= 1000);
 
    }
 
@@ -329,7 +334,7 @@ public class SingleMirrorSoakTest extends SoakTestBase {
 
          for (int i = start; i < start + numberOfMessages; i++) {
             TextMessage message = (TextMessage) consumer.receive(10_000);
-            Assert.assertNotNull(message);
+            assertNotNull(message);
             logger.debug("Received message {}, large={}", message.getIntProperty("i"), message.getBooleanProperty("large"));
             if (message.getIntProperty("i") != i) {
                failed = true;
@@ -348,22 +353,11 @@ public class SingleMirrorSoakTest extends SoakTestBase {
          }
          session.commit();
 
-         Assert.assertFalse(failed);
+         assertFalse(failed);
 
          if (expectEmpty) {
-            Assert.assertNull(consumer.receiveNoWait());
+            assertNull(consumer.receiveNoWait());
          }
       }
    }
-
-   public long getCount(SimpleManagement simpleManagement, String queue) {
-      try {
-         long value = simpleManagement.getMessageCountOnQueue(queue);
-         logger.info("Queue {} count = {}", queue, value);
-         return value;
-      } catch (Exception e) {
-         logger.warn(e.getMessage(), e);
-         return -1;
-      }
-   }
 }
